package table;

import java.util.Stack;

import function.LispFunction;
import sexpression.SExpression;

public class ExecutionContext {

    private static ExecutionContext uniqueInstance = new ExecutionContext();

    public static ExecutionContext getInstance() {
        return uniqueInstance;
    }

    private SymbolTable scope;
    private Stack<LispFunction> functionCalls;
    private boolean recur;

    private ExecutionContext() {
        this.scope = new SymbolTable();
        this.functionCalls = new Stack<>();
        this.clearRecur();
    }

    public SymbolTable getScope() {
        return scope;
    }

    public void setScope(SymbolTable scope) {
        this.scope = scope;
    }

    public void clearContext() {
        this.scope = new SymbolTable();
        this.functionCalls = new Stack<>();
        this.clearRecur();
    }

    public SExpression lookupSymbolValue(String symbolName) {
        for (SymbolTable t = scope; t != null; t = t.getParent())
            if (t.contains(symbolName))
                return t.get(symbolName);

        return null;
    }

    public void pushFunctionCall(LispFunction function) {
        functionCalls.push(function);
    }

    public void popFunctionCall() {
        functionCalls.pop();
    }

    public boolean isInFunctionCall() {
        return !functionCalls.empty();
    }

    public LispFunction getCurrentFunction() {
        return functionCalls.peek();
    }

    public boolean isRecur() {
        return recur;
    }

    public void setRecur() {
        recur = true;
    }

    public void clearRecur() {
        recur = false;
    }

}
